<?php

require '/var/packages/GlacierBackup/target/lib/aws/aws.phar';

use Aws\Glacier\GlacierClient;
use Aes\Glacier\Exception\GlacierException;
use Aes\Glacier\Exception\ResourceNotFoundException;
use Aes\Glacier\Exception\InvalidParameterValueException;
use Aes\Glacier\Exception\LimitExceededException;
use Aes\Glacier\Exception\MissingParameterValueException;
use Aes\Glacier\Exception\RequestTimeoutException;
use Aes\Glacier\Exception\ServiceUnavailableException;
use Aws\Glacier\Model\MultipartUpload\UploadPartGenerator;
use Aws\Common\Enum\Region;
use Guzzle\Http\EntityBody;
use Aws\Common\Hash\TreeHash;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Guzzle\Common\Event;

function SendMsgToClient($msgsock, $msg)
{
	$len = strlen($msg);
	$packOfLen = pack("I", $len);
	socket_write($msgsock, $packOfLen, 4);
	socket_write($msgsock, $msg, $len);
};


function ReceiveMsgFromClient($msgsock)
{
	$len = -1;
	if (false === ($inputStringSize = socket_read($msgsock, 4))) {
		syslog(LOG_ERR, "socket_read() failed: reason: " . socket_strerror(socket_last_error($msgsock)));
		return false;			
	}

	$unpackArray = unpack("I", $inputStringSize);
	$unpackInteger = $unpackArray[1];

	if (false === ($inputString = socket_read($msgsock, $unpackInteger, PHP_NORMAL_READ))) {
		syslog(LOG_ERR, "socket_read() failed: reason: " . socket_strerror(socket_last_error($msgsock)));
		return false;
	}

	return $inputString;
};


function CalculateChecksum($aws, $msgsock) {
	$ret = false;
	if (false===($filePath = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [filePath].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	if (false===($strMultipartSize = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [filePath].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}


	$multipartSize = (int)$strMultipartSize;
	$uploadFileFp = fopen($filePath, "r");
	if ($uploadFileFp) {
		if (0==$multipartSize) {
			$multipartSize = 256*1024*1024;
		}

		try {
			$parts = UploadPartGenerator::factory($uploadFileFp, $multipartSize);
			$treeHash = $parts->getRootChecksum();
		} catch (Exception $e) {
			fclose($uploadFileFp);
			SendMsgToClient($msgsock, $e);
			goto END;
		}

		fclose($uploadFileFp);
		SendMsgToClient($msgsock, "checksum");
		SendMsgToClient($msgsock, $treeHash);
	} else {
		syslog(LOG_ERR, "Can not open file." . $filePath);
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	$ret = true;

END:
	return $ret;
}


function SinglePartUpload($aws, $msgsock) {
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
        }
	if (false===($filePath = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [filePath].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($fileSize = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [fileSize].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	$uploadFileFp = fopen($filePath, "r");
	if (!$uploadFileFp) {
		syslog(LOG_ERR, "Can not open file." . $filePath);
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	try {
		$result = $aws->uploadArchive(array(
			'vaultName' => $vaultName,
			'body' => $uploadFileFp
		));
		fclose($uploadFileFp);
	} catch (Exception $e) {
		fclose($uploadFileFp);
 		SendMsgToClient($msgsock, $e);
		goto END;
	}


	$msg = "archiveID";
	SendMsgToClient($msgsock, $msg);
	$msg = $result->get('archiveId');
	SendMsgToClient($msgsock, $msg);

	$transmittedSize = 0;
	$totalSize = 0;
	$handle = fopen("/tmp/backup/glacierBackupProgress", "r");
	if ($handle) {
		// Read progress from file
		while (false!=($buffer = fgets($handle))) {
			if (0==strncmp($buffer, "transmitted_size=", strlen("transmitted_size="))) {
				sscanf($buffer, "transmitted_size=\"%64[^\"]%*[\"]\n", $transmittedSize);
			}
			if (0==strncmp($buffer, "total_size=", strlen("total_size="))) {
				sscanf($buffer, "total_size=\"%64[^\"]%*[\"]\n", $totalSize);
			}
		}

		fclose($handle);
	}

	$transmittedSize = $transmittedSize + $fileSize;
	if (($fp = fopen("/tmp/backup/glacierBackupProgress.tmp", "w"))) {
		fprintf($fp, "transmitted_size=\"" . $transmittedSize . "\"\ntotal_size=\"" . $totalSize . "\"\n");
		fclose($fp);
		rename("/tmp/backup/glacierBackupProgress.tmp", "/tmp/backup/glacierBackupProgress");
	}

	$ret = true;
END:
	return ret;
};


function MultiPartUploadInit($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($multipartSize = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [multipartSize].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	try {
		$multiupload = $aws->initiateMultipartUpload(array(
			'vaultName' => $vaultName,
			'partSize' => $multipartSize
			));
		SendMsgToClient($msgsock, "uploadID");
		SendMsgToClient($msgsock, $multiupload->get('uploadId'));
	} catch (Exception $e) {
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$ret = true;
END:
	return $ret;
}


function MultiPartUploadAbort($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($uploadID = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [uploadID].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	try {
		$aws->abortMultipartUpload(array(
			'vaultName' => $vaultName,
			'uploadId' => $uploadID
		));
		SendMsgToClient($msgsock, "done");
	} catch (Exception $e) {
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$ret = true;
END:
	return $ret;
}


function MultiPartUploadComplete($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($uploadID = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [uploadID].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($fileSize = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [fileSize].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($filePath = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [filePath].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($treeHash = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [filePath].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	$uploadFileFp = fopen($filePath, "r");
	if (!$uploadFileFp) {
		syslog(LOG_ERR, "Can not open file." . $filePath);
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	try {
		$result = $aws->completeMultipartUpload(array(
			'vaultName' => $vaultName,
			'uploadId' => $uploadID,
			'archiveSize' => $fileSize,
			'checksum' => $treeHash
		));

		fclose($uploadFileFp);
		SendMsgToClient($msgsock, "archiveID");
		SendMsgToClient($msgsock, $result->get('archiveId'));
	} catch (Exception $e) {
		fclose($uploadFileFp);
		SendMsgToClient($msgsock, $e);
		goto END;
	}
	$ret = true;
END:
	return $ret;
}


function MultiPartUpload($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($uploadID = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [uploadID].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($range = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [range].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($partFilePath = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [partFilePath].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($partFileSize = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [partFileSize].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	$uploadFileFp = fopen($partFilePath, "r");
	if (!$uploadFileFp) {
		syslog(LOG_ERR, "Can not open file." . $partFilePath);
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	try {
		$resultUpload = $aws->uploadMultipartPart(array(
			'vaultName' => $vaultName,
			'uploadId' => $uploadID,
			'range' => $range,
			'body' => $uploadFileFp,
		));
		fclose($uploadFileFp);
	} catch (Exception $e) {
		fclose($uploadFileFp);
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	SendMsgToClient($msgsock, "upload_multipart_file_continue_ok");

	$transmittedSize = 0;
	$totalSize = 0;
	$handle = fopen("/tmp/backup/glacierBackupProgress", "r");
	if ($handle) {
		// Read progress from file
		while (false!=($buffer = fgets($handle))) {
			if (0==strncmp($buffer, "transmitted_size=", strlen("transmitted_size="))) {
				sscanf($buffer, "transmitted_size=\"%64[^\"]%*[\"]\n", $transmittedSize);
			}
			if (0==strncmp($buffer, "total_size=", strlen("total_size="))) {
				sscanf($buffer, "total_size=\"%64[^\"]%*[\"]\n", $totalSize);
			}
		}

		fclose($handle);
	}

	$transmittedSize = $transmittedSize + $partFileSize;
	if (($fp = fopen("/tmp/backup/glacierBackupProgress.tmp", "w"))) {
		fprintf($fp, "transmitted_size=\"" . $transmittedSize . "\"\ntotal_size=\"" . $totalSize . "\"\n");
		fclose($fp);
		rename("/tmp/backup/glacierBackupProgress.tmp", "/tmp/backup/glacierBackupProgress");
	}
	$ret = true;
END:
	return $ret;
};


function InitDownloadFile ($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		SendMsgToClient($msgsock, "Fatal error.");
		syslog(LOG_ERR, "Input bad parameter, [archiveID].");
		goto END;
	}
	if (false===($archiveID = ReceiveMsgFromClient($msgsock))) {
		SendMsgToClient($msgsock, "Fatal error.");
		syslog(LOG_ERR, "Input bad parameter, [archiveID].");
		goto END;
	}	
	if (false===($RetrievalByteRange = ReceiveMsgFromClient($msgsock))) {
		SendMsgToClient($msgsock, "Fatal error.");
		syslog(LOG_ERR, "Input bad parameter, [RetrievalByteRange].");
		goto END;
	}

	try {
		if (0==strcmp($RetrievalByteRange, "all")) {
			$result = $aws->initiateJob(array(
				'vaultName' => $vaultName,
				'Type' => 'archive-retrieval',
				'ArchiveId' => $archiveID
			));
		} else {
			$result = $aws->initiateJob(array(
				'vaultName' => $vaultName,
				'Type' => 'archive-retrieval',
				'ArchiveId' => $archiveID,
				'RetrievalByteRange' => $RetrievalByteRange
			));
		}
		$msg = "jobID";
		SendMsgToClient($msgsock, $msg);
		$msg = $result->get('jobId');
		SendMsgToClient($msgsock, $msg);
	} catch (Exception $e) {
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$ret = true;

END:
	return $ret;
};

function DescribeJob ($aws, $msgsock)
{
	$ret = false;
	if (false === ($vaultName = ReceiveMsgFromClient($msgsock))) {
		SendMsgToClient($msgsock, "Fatal error.");
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		goto END;
	}
	if (false === ($jobID = ReceiveMsgFromClient($msgsock))) {
		SendMsgToClient($msgsock, "Fatal error.");
		syslog(LOG_ERR, "Input bad parameter, [localPath].");
		goto END;
	}
	try {
		$result = $aws->describeJob(array(
			'vaultName' => $vaultName,
			'jobId' => $jobID
		));

		$msg = "StatusCode";
		SendMsgToClient($msgsock, $msg);
		$msg = $result->get('StatusCode');
		SendMsgToClient($msgsock, $msg);
		$msg = "Completed";
		SendMsgToClient($msgsock, $msg);
		if ($result->get('Completed')) {
			$msg = "true";
		} else {
			$msg = "false";
		}
		SendMsgToClient($msgsock, $msg);
	} catch (Exception $e) {
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$ret = true;
END:
	return $ret;
};

function StartDownloadFile ($aws, $msgsock, $partSize)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($jobID = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [jobID].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($byteRange = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [byteRange].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($partFile = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [partFile].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	$fp = fopen($partFile, "w+");
	if (!$fp) {
		syslog(LOG_ERR, "Can not open file." . $partFile);
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	try {
		if (0==strcmp($byteRange, "all")) {
			$result = $aws->getJobOutput(array(
				'vaultName' => $vaultName,
				'jobId' => $jobID,
				'saveAs' => $fp
			));
		} else {
			$result = $aws->getJobOutput(array(
				'vaultName' => $vaultName,
				'jobId' => $jobID,
				'range' => $byteRange,
				'saveAs' => $fp
			));
		}

		fclose($fp);
		$msg = "download_complete";
		SendMsgToClient($msgsock, $msg);
	} catch (Exception $e) {
		fclose($fp);
		SendMsgToClient($msgsock, $e);
		goto END;
        }

	$ret = true;
END:
	return $ret;
};


function RetrieveInventoryStart($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($jobID = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [jobID].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($localFile = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [localFile].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	$fp = fopen($localFile, "w+");
	if (!$fp) {
		syslog(LOG_ERR, "Can not open file." . $localFile);
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	try {
		$result = $aws->getJobOutput(array(
			'vaultName' => $vaultName,
			'jobId' => $jobID,
			'saveAs' => $fp
		));

		fclose($fp);
		$msg = "retrieve_complete";
		SendMsgToClient($msgsock, $msg);
	} catch (Exception $e) {
		fclose($fp);
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$ret = true;
END:
	return $ret;
}


function ListVaults ($aws, $msgsock)
{
	$ret = false;
	$marker = "";
	$totalCount = 0;

	if (false===($listMaxNumber = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [listMaxNumber].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	while (1) {
		if (strlen($marker)>0) {
			try {
				$result = $aws->listVaults(array('marker' => $marker));
			} catch (Exception $e) {
				SendMsgToClient($msgsock, $e);
				goto END;
			}
		} else {
			try {
				$result = $aws->listVaults(array());
			} catch (Exception $e) {
				SendMsgToClient($msgsock, $e);
				goto END;
			}
		}

		$vaultList = $result->get('VaultList');
		$count = count($vaultList);

		for ($i = 0; $i < $count; $i++) {
			$msg = $vaultList[$i]["VaultARN"];
			SendMsgToClient($msgsock, $msg);
		}

		$totalCount = $totalCount + $count;
		if (($listMaxNumber != -1) && ($totalCount >= $listMaxNumber)) {
			break;
		}

		$marker = $result->get('Marker');
		if (0==strlen($marker)) {
			break;
		}
	}
	$msg = "list_vault_done";
	SendMsgToClient($msgsock, $msg);

	$ret = true;

END:

	return $ret;
};


function DeleteArchive ($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	if (false===($archiveID = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [archiveID].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}
	try {
		$aws->deleteArchive(array(
			'vaultName' => $vaultName,
			'archiveId' => $archiveID
		));
	} catch (Exception $e) {
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$msg = "delete_archive_done";
	SendMsgToClient($msgsock, $msg);

	$ret = true;
END:
	return $ret;
}


function CreateVault ($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	try {
		$aws->createVault(array(
			'vaultName' => $vaultName
		));
	} catch (Exception $e) {
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$msg = "create_vault_done";
	SendMsgToClient($msgsock, $msg);
	$ret = true;
END:
	return $ret;
}


function DescVault($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	try {
		$result = $aws->describeVault(array(
			'vaultName' => $vaultName
		));

		$vaultVRN = $result->get('VaultARN');
		SendMsgToClient($msgsock, $vaultVRN);
	} catch (Exception $e) {
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$ret = true;
END:
	return $ret;
}


function DeleteVault($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	try {
		$aws->deleteVault(array(
			'vaultName' => $vaultName
		));
		SendMsgToClient($msgsock, "done");
	} catch (Exception $e) {
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$ret = true;
END:
	return $ret;
}


function RetrieveInventoryInit($aws, $msgsock)
{
	$ret = false;
	if (false===($vaultName = ReceiveMsgFromClient($msgsock))) {
		syslog(LOG_ERR, "Input bad parameter, [vaultName].");
		SendMsgToClient($msgsock, "Fatal error.");
		goto END;
	}

	try {
		$result = $aws->initiateJob(array(
			'vaultName' => $vaultName,
			'Type' => "inventory-retrieval" 
		));

		SendMsgToClient($msgsock, "jobID");
		$jobID = $result->get('jobId');
		SendMsgToClient($msgsock, $jobID);
	} catch (Exception $e) {
		SendMsgToClient($msgsock, $e);
		goto END;
	}

	$ret = true;

END:
	return $ret;
}

$socketID = $argv[1];
$region = $argv[2];
$partSize = "4194304";
$accessKey = getenv("AWS_ACCESS_KEY_ID");
$secretKey = getenv("AWS_SECRET_ACCESS_KEY");
$scheme = getenv("AWS_SCHEME");

class SynoUserAgentListener implements EventSubscriberInterface
{
	public static function getSubscribedEvents()
	{
		return array('command.before_send' => 'onBeforeSend');
	}
	public function onBeforeSend(Event $event)
	{
		$command = $event['command'];
		$request = $command->getRequest();
		$userAgent = (string) $request->getHeader('User-Agent');
		$request->setHeader('User-Agent', getenv("GLACIER_USER_AGENT") . " " . $userAgent);
	}
}

$aws = GlacierClient::factory(array(
	'key'    => $accessKey,
	'secret' => $secretKey,
	'region' => $region,
	'scheme' => $scheme,
	'ssl.certificate_authority' => '/var/packages/GlacierBackup/target/lib/aws/cacert.pem',
	'curl.options' => array('CURLOPT_CONNECTTIMEOUT' => 60, 'CURLOPT_LOW_SPEED_LIMIT' => 1, 'CURLOPT_LOW_SPEED_TIME' => 1800)

	));

$aws->addSubscriber(new SynoUserAgentListener());

if (false===($sock = socket_create(AF_UNIX, SOCK_STREAM, 0))) {
	syslog(LOG_ERR, "socket_create() failed: reason: " . socket_strerror(socket_last_error()));
	goto END;
}

if (false===socket_bind($sock, $socketID)) {
	syslog(LOG_ERR, "socket_bind() failed: reason: " . socket_strerror(socket_last_error($sock)));
	goto END;
}

if (false===socket_listen($sock, 1)) {
	syslog(LOG_ERR, "socket_listen() failed: reason: " . socket_strerror(socket_last_error($sock)));
	goto END;
}

do {
	if (false===($msgsock = socket_accept($sock))) {
		syslog (LOG_ERR,"socket_accept() failed: reason: " . socket_strerror(socket_last_error($sock)));
		break;
	}

	/* Send instructions. */
	$msg = "connect_succeed";
	SendMsgToClient($msgsock, $msg);
	do {
		$inputString = ReceiveMsgFromClient($msgsock);
		if (false===$inputString) {
			continue;
		}

		/* Stop Server Command
		*  Receive : stop_server
		*  Ack: stop_server_succeed
		*/
		if ($inputString == 'stop_server') {
			$msg = "stop_server_succeed";
			SendMsgToClient($msgsock, $msg);
			goto END;
		}

		/* Upload single part file Command
		*  Receive 1 : upload_file
		*  Receive 2 : filePath
		*  Receive 3 : fileSize
		*  Ack: Archive ID
		*/
		if (0==strcmp($inputString, "upload_single_file")) {
			SinglePartUpload($aws, $msgsock);
			continue;
		}

		/* Initate multipart upload.
		*  Receive 1 : upload_multipart_file_initate
		*  Ack: Upload ID
		*/
		if (0==strcmp($inputString, "upload_multipart_file_initate")) {
			MultiPartUploadInit($aws, $msgsock);
			continue;
		}

		/* Continue multipart upload.
		*  Receive 1 : upload_multipart_file_continue
		*  Receive 2 : uploadID
		*  Receive 3 : range
		*  Receive 4 : partFilePath
		*  Receive 5 : partFileSize
		*  Ack: archiveID
		*/
		if (0==strcmp($inputString, "upload_multipart_file_continue")) {
			MultiPartUpload($aws, $msgsock);
			continue;
		}

		if (0==strcmp($inputString, "upload_multipart_file_complete")) {
			MultiPartUploadComplete($aws, $msgsock);
			continue;
		}

		if (0==strcmp($inputString, "upload_multipart_file_abort")) {
			MultiPartUploadAbort($aws, $msgsock);
			continue;
		}

		/* Download file command (Download job output)
		*  Receive 1 : start_download_file
		*  Reveive 2 : jobID
		*  Receive 3 : fileSize
		*  Receive 4 : localPath
		*  Ack: download_complete
		*/
		if (0==strcmp($inputString, "start_download_file")) {
			StartDownloadFile($aws, $msgsock, $partSize);
			continue;
		}

		/* Describe job
		*  Receive 1 : describe_job
		*  Receive 2 : jobID
		*  Ack 1: StatusCode
		*  Ack 2: status
		*  Ack 3: Completed 
		*  Ack 4: ture or false
		*/
		if (0==strcmp($inputString, "describe_job")) {
			DescribeJob($aws, $msgsock);
			continue;
		}

		/* Init a download job
		*  Receive 1 : init_download_file 
		*  Receive 2 : archiveID
		*  Ack 1: jobID
		*/
		if (0==strcmp($inputString, "init_download_file")) {
			InitDownloadFile($aws, $msgsock);
			continue;
		}

		/* List vault
		*  Receive 1 : list_vault 
		*  Ack N: Vault list 
		*/
		if (0==strcmp($inputString, "list_vault")) {
			ListVaults($aws, $msgsock);
			continue;
		}

		/* Delete archive
		*  Receive 1 : delete_archive
		*  Receive 2 : archive id
		*  Ack 1 : delete_archive_done
 		*/
		if (0==strcmp($inputString, "delete_archive")) {
			DeleteArchive($aws, $msgsock);
			continue;
		}

		/* Create vault
		*  Receive 1 : create_vault
		*  Receive 2 : vault name
		*  Ack 1: create_vault_done
		*/
		if (0==strcmp($inputString, "create_vault")) {
			CreateVault($aws, $msgsock);
			continue;
		}

		/* Desc vault
		*  Receive 1 : desc_vault
		*  Receive 2 : vault name
		*  Ack 1: Arn of vault
		*/
		if (0==strcmp($inputString, "desc_vault")) {
			DescVault($aws, $msgsock);
			continue;
		}

		/* Delete vault
		*  Receive 1 : delete_vault
		*  Ack 1: done
		*/
		if (0==strcmp($inputString, "delete_vault")) {
			DeleteVault($aws, $msgsock);
			continue;
		}

		/* Init Retrieve Inventory
		*  Receive 1 : retrieve_inventory_init
		*  Receive 2 : vault name
		*  Ack 1: jobID
		*/
		if (0==strcmp($inputString, "retrieve_inventory_init")) {
			RetrieveInventoryInit($aws, $msgsock);
			continue;
		}

		/* Start Retrieve Inventory
		*  Receive 1 : retrieve_inventory_start
		*  Receive 2 : vault name
		*  Receive 3 : job id
		*  Receive 4 : local path
		*  Ack 1: download_complete
		*/
		if (0==strcmp($inputString, "retrieve_inventory_start")) {
			RetrieveInventoryStart($aws, $msgsock);
			continue;
		}

		/* Start Retrieve Inventory
		*  Receive 1 : calculate_checksum
		*  Receive 2 : file path
		*  Ack 1: checksum
		*/
		if (0==strcmp($inputString, "calculate_checksum")) {
			CalculateChecksum($aws, $msgsock);
			continue;
		}

	} while (true);

	socket_close($msgsock);

} while (true);


END:

socket_close($sock);

return;
                                                                                                                                                                                                                        
?>
