#!/usr/bin/env php
<?php
class Arg {
	protected $name = null;
	protected $parser = null;
	protected $optional = false;

	function __construct($name, $parser = null) {
		if ($name[0] == '?') {
			$this->name = substr($name, 1);
			$this->optional = true;
		} else {
			$this->name = $name;
		}
		$this->parser = $parser;
	}
	function __toString() {
		return $this->name;
	}
	function parse($arg) {
		if ($this->parser) {
			return call_user_func($this->parser, $arg);
		}

		return $arg;
	}
	function is_optional() {
		return $this->optional;
	}
	static function open_for_read($arg) {
		$fp = fopen($arg, "r");
		return $fp;
	}
	static function parse_args(&$argv, $argconf, $usage = null, $usage_arg = null) {
		$data = array();
		foreach ($argconf as $a) {
			$arg = array_shift($argv);

			if (! $a instanceof Arg) {
				$a = new Arg($a);
			}

			if (!$arg) {
				if (!$a->is_optional()) {
					printf("parse command \"$usage_arg\" error\n");
					$usage && $usage($usage_arg);
					exit(1);
				} else {
					$a->parse($arg);
					continue;
				}
			}

			$data["$a"] = $a->parse($arg);
		}
		if (count($argv)) {
			printf("parse command \"$usage_arg\" error\n");
			$usage && $usage($usage_arg);
			exit(1);
		}
		return $data;
	}
}

function get_aws_result($cmd, $data) {
	global $aws, $cset;

	echo "---- $cmd ----\n";
	$result = $aws->$cmd($data);

	$keys = $cset[$cmd]['keys'];
	foreach ($keys as $k) {
		printf("%30s: %s\n", $k, $result->get($k));
	}
	return $result;
}
function get_aws_result_list($cmd, $data) {
	global $aws, $cset;

	echo "---- $cmd ----\n";
	$result = $aws->$cmd($data);

	$list_key = $cset[$cmd]['list_key'];
	$keys = $cset[$cmd]['keys'];
	$i = 0;
	foreach ($result->get($list_key) as $item) {
		echo "[$i]\n";
		foreach ($keys as $k) {
			printf("%30s: %s\n", $k, $item[$k]);
		}
		$i += 1;
	}
	return $result;
}

function tree_hash($cmd, $data) {
	echo "---- $cmd ----\n";
	$fp = $data['FILE'];

	$eb = EntityBody::factory($fp);
	$th = TreeHash::fromContent($eb);
	$treeHash = $th->getHash();

	echo $treeHash."\n";
	return $treeHash;
}
function make_chunk($fp, $start, $size, &$chunk_fname) {
	$chunk_fname = sprintf("test_aws.tmp.$start");
	$chunk_fp = fopen($chunk_fname, "w+");

	$remain_size = $size;
	while ($remain_size > 0) {
		$read_size = min($remain_size, 1024*1024);
		$buf = fread($fp, $read_size);
		if ('' === $buf) {
			rewind($chunk_fp);
			return $chunk_fp;
		}
		$ret = fwrite($chunk_fp, $buf);

		if ($ret === false || $ret < 0) {
			throw new Exception("make chunk failed");
			return false;
		}
		$remain_size -= $ret;
	}

	rewind($chunk_fp);
	return $chunk_fp;
}
function upload($cmd, $data) {
	global $aws;
	global $cset;
	global $aws_config;

	echo "---- $cmd ----\n";
	$part_size = 4 * 1024 * 1024; // 4194304; // 4 MB
	$fpath = $data['FILE'];
	$fsize = filesize($fpath);

	if ($fsize <= $part_size) {
		$data['body'] = fopen($fpath, "r");
		return get_aws_result('uploadArchive', $data);
	}

	$dup_data = $data;
	$dup_data['partSize'] = $part_size;
	$result = get_aws_result('initiateMultipartUpload', $dup_data);
	$data['uploadId'] = $result->get('uploadId');

	$fp = fopen($fpath, "r");
	$start = 0;
	$remain_size = $fsize;
	rewind($fp);
	while (0 < $remain_size) {
		$upload_size = min($part_size, $remain_size);

		$end = $start + $upload_size - 1;
		$content = fread($fp, $upload_size);
		// solution 1. re create client
		//$aws = GlacierClient::factory($aws_config);
		//
		// solution 2. use volume, not temp
		//$chunk_fname = '';
		//$chunk_fp = make_chunk($fp, $start, $upload_size, &$chunk_fname);

		$dup_data = $data;
		$dup_data['range'] = sprintf('bytes %d-%d/*', $start, $end);
		$dup_data['body'] = $content;
		$result = get_aws_result('uploadMultipartPart', $dup_data);

		$remain_size -= $upload_size;
		$start += $part_size;

		//fclose($chunk_fp);
		//unlink($chunk_fname);
		echo memory_get_usage(true)."\n";
	}
	fclose($fp);

	$fp = fopen($fpath, "r");
	$treeHash = tree_hash('tree_hash', array(
		'FILE' => $fp
	));

	$result = get_aws_result('completeMultipartUpload', array(
		'vaultName' => $data['vaultName'],
		'uploadId' => $data['uploadId'],
		'archiveSize' => $fsize,
		'checksum' => $treeHash
	));
}

$cset = array(
	// ---- vault operations ----
	'createVault' => array(
		'args' => array('vaultName'),
		'handler' => 'get_aws_result',
		'keys' => array('location')
	),
	'deleteVault' => array(
		'args' => array('vaultName'),
		'handler' => 'get_aws_result',
		'keys' => array()
	),
	'describeVault' => array(
		'args' => array('vaultName'),
		'handler' => 'get_aws_result',
		'keys' => array(
			"VaultARN", "VaultName", "CreationDate",
			"LastInventoryDate", "NumberOfArchives", "SizeInBytes"
		)
	),
	'listVaults' => array(
		'args' => array('?marker'),
		'handler' => 'get_aws_result_list',
		'list_key' => 'VaultList',
		'keys' => array(
			"VaultName", "CreationDate", "LastInventoryDate",
		       	"NumberOfArchives", "SizeInBytes", "VaultARN"
		)
	),

	// ---- upload ----
	'initiateMultipartUpload' => array(
		'args' => array('vaultName', 'partSize', '?archiveDescription'),
		'handler' => 'get_aws_result',
		'keys' => array('location', 'uploadId')
	),
	'uploadMultipartPart' => array(
		'args' => array('vaultName', 'uploadId', new Arg('body', 'Arg::open_for_read')),
		'handler' => 'get_aws_result',
		'keys' => array('checksum')
	),
	'listMultipartUploads' => array(
		'args' => array('vaultName'),
		'handler' => 'get_aws_result_list',
		'list_key' => 'UploadsList',
		'keys' => array(
			"MultipartUploadId", "VaultARN",
			"ArchiveDescription", "PartSizeInBytes", "CreationDate"
		)
	),
	'listParts' => array(
		'args' => array('vaultName', 'uploadId'),
		'handler' => 'get_aws_result_list',
		'list_key' => 'Parts',
		'keys' => array(
			"RangeInBytes", "SHA256TreeHash"
		)
	),
	'abortMultipartUpload' => array(
		'args' => array('vaultName', 'uploadId'),
		'handler' => 'get_aws_result',
		'keys' => array()
	),
	'completeMultipartUpload' => array(
		'args' => array('vaultName', 'uploadId', '?checksum'),
		'handler' => 'get_aws_result',
		'keys' => array('checksum', 'archiveId')
	),
	'uploadArchive' => array(
		'args' => array('vaultName', new Arg('body', 'Arg::open_for_read'), '?archiveDescription'),
		'handler' => 'get_aws_result',
		'keys' => array('checksum', 'archiveId')
	),

	// ---- delete archive ----
	'deleteArchive' => array(
		'args' => array('vaultName', 'archiveId'),
		'handler' => 'get_aws_result',
		'keys' => array()
	),

	// ---- job operations ----
	'initiateJob' => array(
		'args' => array('vaultName', 'ArchiveId'),
		'handler' => 'get_aws_result',
		'keys' => array('location', 'jobId')
	),
	'listJobs' => array(
		'args' => array('vaultName'),
		'handler' => 'get_aws_result_list',
		'list_key' => 'JobList',
		'keys' => array(
			"JobId", "Action", "ArchiveId", "CreationDate",
			"Completed", "StatusCode", "StatusMessage",
		       	"ArchiveSizeInBytes", "InventorySizeInBytes", "CompletionDate"
		)
	),
	'describeJob' => array(
		'args' => array('vaultName', 'jobId'),
		'handler' => 'get_aws_result',
		'keys' => array('StatusCode', 'Completed')
	),
	'getJobOutput' => array(
		'args' => array('vaultName', 'jobId', 'saveAs', '?range'),
		'handler' => 'get_aws_result',
		'keys' => array('checksum', 'status', 'contentRange', 'acceptRanges', 'contentType', 'archiveDescription', 'body')
	),

	// ---- other combo ----
	'tree_hash' => array(
		'args' => array(new Arg('FILE', 'Arg::open_for_read'), '?offset', '?size'),
		'handler' => 'tree_hash',
	),
	'upload' => array(
		'args' => array('vaultName', 'FILE'),
		'handler' => 'upload',
	),
	'list_all_vaults' => array(
		'args' => array(),
		'handler' => function ($cmd, $data) {
			global $aws, $cset;

			foreach (array("keys", "list_key") as $key) {
				$cset["listVaults"][$key] = $cset["list_all_vaults"][$key];
			}

			do {
				$result = get_aws_result_list("listVaults", $data);
				echo "marker: ".$result->get("Marker")."\n";
				$data["marker"] = $result->get("Marker");
			} while ($result->get("Marker"));
		},
		'list_key' => 'VaultList',
		'keys' => array('VaultARN'),
	),
);

function usage($cmd = null) {
	global $cset;

	if ($cmd !== null && array_key_exists($cmd, $cset)) {
		$v =& $cset[$cmd];
		if (count($v['args'])) {
			printf("\t%s [%s]\n", $cmd, implode('] [', $v['args']));
		} else {
			printf("\t%s\n", $cmd);
		}
	} else {
		foreach ($cset as $cmd => $v) {
			if (count($v['args'])) {
				printf("\t%s [%s]\n", $cmd, implode('] [', $v['args']));
			} else {
				printf("\t%s\n", $cmd);
			}
		}
	}
}

$progname = array_shift($argv);
$cmd = array_shift($argv);

if ($cmd == 'extract') {
	$out_dir = "aws-sdk";
	if (!file_exists($out_dir)) {
		mkdir($out_dir);
	}
	$phar = new Phar('aws.phar');
	$phar->extractTo($out_dir);
	exit(0);
}

error_reporting(E_ALL);

$debug = '';
if ('fake' === $debug) {
	require_once 'fake_aws.php';

	$lib_prefix = './aws-sdk';
	define('AWS_PHAR', true);

	require_once $lib_prefix.'/vendor/symfony/class-loader/Symfony/Component/ClassLoader/UniversalClassLoader.php';

	$classLoader = new Symfony\Component\ClassLoader\UniversalClassLoader();
	$classLoader->registerNamespaces(array(
	    'Guzzle'   => $lib_prefix.'/vendor/guzzle/guzzle/src',
	    'Symfony\\Component\\EventDispatcher' => $lib_prefix.'/vendor/symfony/event-dispatcher',
	    'Doctrine' => $lib_prefix.'/vendor/doctrine/common/lib',
	    'Monolog'  => $lib_prefix.'/vendor/monolog/monolog/src'
	));
	$classLoader->register();
} else if ('lib' === $debug) {
	//$lib_prefix = '/mnt/source/GlacierBackup/lib/aws/aws-sdk';
	$lib_prefix = './aws-sdk';
	define('AWS_PHAR', true);

	require_once $lib_prefix.'/vendor/symfony/class-loader/Symfony/Component/ClassLoader/UniversalClassLoader.php';

	$classLoader = new Symfony\Component\ClassLoader\UniversalClassLoader();
	$classLoader->registerNamespaces(array(
	    'Aws'      => $lib_prefix.'/src',
	    'Guzzle'   => $lib_prefix.'/vendor/guzzle/guzzle/src',
	    'Symfony\\Component\\EventDispatcher' => $lib_prefix.'/vendor/symfony/event-dispatcher',
	    'Doctrine' => $lib_prefix.'/vendor/doctrine/common/lib',
	    'Monolog'  => $lib_prefix.'/vendor/monolog/monolog/src'
	));
	$classLoader->register();
} else {
	//$lib_prefix = '/var/packages/GlacierBackup/target/lib/aws';
	$lib_prefix = '.';
	require_once $lib_prefix.'/aws.phar';
}

use Aws\Glacier\GlacierClient;
use Aws\Common\Hash\TreeHash;
use Guzzle\Http\EntityBody;
use Guzzle\Common\Event;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class SynoUserAgentListener implements EventSubscriberInterface
{
	public static function getSubscribedEvents()
	{
		return array('command.before_send' => 'onBeforeSend');
	}
	public function onBeforeSend(Event $event)
	{
		$command = $event['command'];
		$request = $command->getRequest();
		$userAgent = (string) $request->getHeader('User-Agent');
		$request->setHeader('User-Agent', $userAgent . " hihi");
	}
}

if (!array_key_exists($cmd, $cset)) {
	printf("Usage:\n");
	usage();
	exit(1);
}
$cmdconf =& $cset[$cmd];

$data = Arg::parse_args(&$argv, $cmdconf['args'], 'usage', $cmd);

$aws_config = array(
	'key' => getenv("AWS_ACCESS_KEY_ID"),
	'secret' => getenv("AWS_SECRET_ACCESS_KEY"),
	'region' => getenv("AWS_REGION"),
	'ssl.certificate_authority' => 'cacert.pem',
	'curl.options' => array(
		'CURLOPT_CONNECTTIMEOUT' => 60,
		'CURLOPT_VERBOSE' => true,
	),
);
$aws = GlacierClient::factory($aws_config);

$aws->addSubscriber(new SynoUserAgentListener());

echo "in region: ".getenv("AWS_REGION")."\n";
echo "memory_limit: ".ini_get('memory_limit')."\n";

$cset[$cmd]['handler']($cmd, $data, $cmdconf);

echo "Peak memory usage for the script (in bytes): ".memory_get_peak_usage()."\n";
?>
